package editor;
import java.util.Iterator;

import javax.xml.namespace.NamespaceContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.SchemaFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Populates both the horizontal and vertical toolbars described in the XML configuration file.
 * The set of both the horizontal and vertical toolbars will be known as "botonera". 
 * @author hqcasanova
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class CustomisedUI {
    private Toolbar horizBar;	//comprises all buttons of the horizontal toolbar
    private Toolbar vertBar;	//comprises all buttons of the vertical toolbar
    private Document xmlDoc;	//holds the DOM tree
    private String sXMLfile;    //XML configuration file name 
    private XPath xpath;		//holds XPATH expressions
    
    /**
     * Customised constructor. Traverses the XML file to process all described buttons and, as 
     * it finds each one, it is added to the appropriate toolbar. 
     * @param sXMLfile Path to the button configuration XML file.
     * @param sXSDfile Path to the XML's schema file.
     */
	public CustomisedUI (String sXMLfile, String sXSDfile) throws Exception {
		this.sXMLfile = sXMLfile;		  	
		
		    //Sets up the XML validation and reading infrastructure
		    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		    factory.setValidating (false);
		    factory.setNamespaceAware (true);
		    SchemaFactory schemaFactory = SchemaFactory.newInstance ("http://www.w3.org/2001/XMLSchema");
		    factory.setSchema (schemaFactory.newSchema (new Source[] {new StreamSource (sXSDfile)}));
			DocumentBuilder docParser = factory.newDocumentBuilder();
			docParser.setErrorHandler (new XMLErrorHandler());
	        xmlDoc = docParser.parse (sXMLfile);
	        
	        //Sets up the infrastructure for XPATH search
	        NamespaceContext ns = new NamespaceContext() {
	           public String getNamespaceURI (String prefix) {
	              String uri;
	        
	              if (prefix.equals ("def"))
	                 uri = "http://www.iter.es/ns";
	              else uri = null;
	              return uri;
	           }
	           @SuppressWarnings("rawtypes")
			   public Iterator getPrefixes (String val) {return null;}
	           public String getPrefix (String uri) {return null;}
	        };
	        xpath = XPathFactory.newInstance().newXPath();
	        xpath.setNamespaceContext (ns);	        
	          
	        //Gets the button list
	        XPathExpression xpathExpr = xpath.compile ("/def:botonera/def:horizontal/*");
	        NodeList listaBotonesHoriz = (NodeList) xpathExpr.evaluate (xmlDoc, XPathConstants.NODESET);
	        xpathExpr = xpath.compile ("/def:botonera/def:vertical/*");
	        NodeList listaBotonesVert = (NodeList) xpathExpr.evaluate (xmlDoc, XPathConstants.NODESET);
	        	        
	        //Builds both toolbars.
	        horizBar = new Toolbar (listaBotonesHoriz, false, true);
	        vertBar = new Toolbar (listaBotonesVert, true, false);
		 
    }
	
	/**
	 * Retrieves the horizontal toolbar as per XML description
	 * @return Button array object representing the toolbar
	 */
	public Toolbar getHorizBar () {
	   return horizBar;
	}
	
	/**
	 * Retrieves the vertical toolbar as per XML description
	 * @return Button array object representing the toolbar
	 */
	public Toolbar getVertBar () {
	   return vertBar;
	}
	
	
	
	/**
	 * Reads the contents of an XML element whose name has been provided.
	 * @param sElementName Element name.
	 * @return String with the element's content.
	 */
	public String readElement (String sElementName) {
	   String sContent = null;
	   	   	   
	   try {
		   XPathExpression xpathExpr = xpath.compile ("//def:" +sElementName+ "/text()");
		   sContent = (String) xpathExpr.evaluate (xmlDoc, XPathConstants.STRING);
		   if ((sContent == null) || (sContent.isEmpty()))
			  throw new Exception ("El elemento no contiene texto o no existe."); 
		   return sContent;
	   } catch (Exception e) {
    	   	   System.out.println ("Ha habido algún problema al leer el elemento " +sElementName+ ": " +e);
       }
	   return null;
	}
	
	/**
	 * Stores the DOM tree to an XML file. 
	 * @throws Exception Any exception arising from the saving process.
	 */
	private void save() throws Exception {
		  
	    //Sets up the transformer factory responsible for DOM's conversion into a file
		TransformerFactory transFact = TransformerFactory.newInstance();
		Transformer trans = transFact.newTransformer();
		trans.setOutputProperty ("indent", "yes");
	      
		//Saving...
		Source fuente = new DOMSource (xmlDoc); 
	    Result destino = new StreamResult (sXMLfile); 
	    trans.transform (fuente, destino); 
	}
	
	/**
	 * Changes the content of a given XML element
	 * @param sName Element's name
	 * @param sContent Element's content
	 */
	public void writeElement (String sName, String sContent) {
		Element element;      	   	
		
		   try {
			   XPathExpression xpathExpr = xpath.compile ("//def:" +sName);
			   element = (Element) xpathExpr.evaluate (xmlDoc, XPathConstants.NODE);
			   if (element == null)
				  throw new Exception ("El elemento no contiene texto o no existe."); 
			   element.setTextContent (sContent);
			   save();
		   } catch (Exception e) {
	    	   	   System.out.println ("Ha habido algún problema al escribir contenido del elemento " +sName+ ": " +e);
		   }
	}
}