package editor;

import gui.CanvasWindow;
import gui.ImageArray;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Toolkit;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;

import utils.Global;


/**
 * Sets up the Swing environment and launches the editor's main window. During the editor's loading
 * process, a splash screen with a progress bar is shown for purposes of user feedback.
 * @author hqcasanova
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class Launcher {
	public static void main(String[] args) {
		final JProgressBar progress;
		final ImageArray logoImg, creditsImg;
		final CanvasWindow splash;	
								
		//Default values for various swing parameters
		UIManager.put ("ToolTip.background", Global.TOOLTIP_BACKGROUND);
		UIManager.put ("ToolTip.font", Global.INTERFACE_FONT);
		UIManager.put ("Label.font", Global.INTERFACE_FONT);
		UIManager.put ("Button.font", Global.INTERFACE_FONT);
		UIManager.put ("Button.margin", Global.BUTTON_MARGINS);
		UIManager.put ("ComboBox.font", Global.INTERFACE_FONT);
		UIManager.put ("ColorChooser.font", Global.INTERFACE_FONT);
		UIManager.put ("ColorChooser.swatchesSwatchSize", new Dimension (Global.SWATCH_SIZE, Global.SWATCH_SIZE));
		UIManager.put ("PopupMenu.font", Global.INTERFACE_FONT);
		UIManager.put ("TextField.font", Global.INTERFACE_FONT);
		UIManager.put ("OptionPane.font", Global.INTERFACE_FONT);
		UIManager.put ("List.font", Global.INTERFACE_FONT);
		UIManager.put ("ProgressBar.font", Global.INTERFACE_FONT);
		UIManager.put ("TitledBorder.font", Global.INTERFACE_FONT);
		UIManager.put ("List.selectionBackground", Global.DEFAULT_HIGHLIGHT);
		UIManager.put ("TextPane.selectionBackground", Global.DEFAULT_HIGHLIGHT);
								
		//Sets look & feel to Nimbus if available. If so, fix defaults.
		try {
		    for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
		        if ("Nimbus".equals (info.getName())) {
		           UIManager.setLookAndFeel (info.getClassName());
		           UIDefaults LFDefaults = UIManager.getLookAndFeelDefaults();
		           LFDefaults.put ("defaultFont", Global.INTERFACE_FONT);
		           LFDefaults.put ("Button.contentMargins", Global.BUTTON_MARGINS);
		           LFDefaults.put ("Button[MouseOver].backgroundPainter", LFDefaults.get("Button[Focused+MouseOver].backgroundPainter"));
		           LFDefaults.put ("List[Selected].textBackground", Global.DEFAULT_HIGHLIGHT);
		           LFDefaults.put ("nimbusSelection", Global.DEFAULT_HIGHLIGHT);
		           LFDefaults.put ("nimbusFocus", Global.DEFAULT_HIGHLIGHT);
		           LFDefaults.put ("info", Global.TOOLTIP_BACKGROUND);
		           LFDefaults.put ("Panel.background", Global.PANEL_BACKGROUND);
		           LFDefaults.put ("OptionPane.background", Global.PANEL_BACKGROUND);
		           LFDefaults.put ("ColorChooser.background", Global.PANEL_BACKGROUND);
		           LFDefaults.put ("ScrollPane[Enabled].borderPainter", LFDefaults.get("ScrollPane[Enabled+Focused].borderPainter"));
		           break;
		        }
		    }
		} catch (Exception e) {
		        //Just load Java's native Look & Feel
		}
				
		//Configures the progress bar
		progress = new JProgressBar();
	    progress.setStringPainted (true);
		progress.setMaximum (19);
		
		//Sets up the splash window and displays it
		splash = new CanvasWindow (null, null, null, null, false, JFrame.DO_NOTHING_ON_CLOSE, 
						           Global.FAVICON_ED, false);
		splash.getWindow().setAlwaysOnTop (true);
		splash.getCanvas().setBackground (Color.decode("#007EED"));
		logoImg = new ImageArray (Global.SPLASH_IMG, Global.IMG_FOLDER + Global.SEPARADOR_SYS, Global.PICT_EXT);
		logoImg.resize (Global.SPLASH_WIDTH, -1);
		creditsImg = new ImageArray (Global.SPLASH_CREDITS, Global.IMG_FOLDER + Global.SEPARADOR_SYS, Global.PICT_EXT);
		creditsImg.resize (Global.SPLASH_WIDTH, -1);
		splash.populate (new JLabel (logoImg.toIcon()), progress, new JLabel (creditsImg.toIcon()));
		splash.display (null);
		splash.busy (true);
			
		//Print out default font metrics
		System.out.println ("Detected DPI: " +Toolkit.getDefaultToolkit().getScreenResolution());
		System.out.println ("Default font size value: " +Global.FONT_SIZE);
		
		//Sets up the editor's main window within Swing's event dispatching framework.
		SwingUtilities.invokeLater (new Runnable() {
		     public void run() {
			   	  new Editor (Global.LANG.getString ("editorTitulo"), 14,
			   			      UIManager.getLookAndFeel().getID(), progress, splash);
			 }
		});			
	}
}
