package gui;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Window;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JDialog;

/**
 * Builds a dialogue with a given canvas. Fine features such as the dialogue's title 
 * or the close button are configurable.
 * @author hqcasanova
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class CanvasDialogue extends Canvas {
    private JDialog dialogue;				
    private WindowAdapter adaptador;	//closing event managing class
            
    /**
     * Customised constructor. Sets the properties of the dialogue's window and canvas 
     * @param sTitle New dialogue's title
     * @param owner Dialogue's owner
     * @param bgColorTopPanel Background color for the top area. A null value leaves
     * the default color.	 
     * @param bgColorBottomPanel Background color for the bottom area. A null value leaves
     * the default color.	 
     * @param centreColor Background color for the centre area. A null value leaves
     * the default color.	 
     * @param bChangeDim Is the dialogue resizable?
     * @param exitMode What happens when the X button is pressed?
     * @param sFavicon Dialogue's favicon path
     * @param bRemoveButtons Remove X and minimise buttons (effective only if using native L&F)
     * @param bRoundedTop True if top panel are to have rounded corners.
     * @param bRoundedBottom True if bottom panel are to have rounded corners. 
     */
	public CanvasDialogue (String sTitle, Window owner, Color colorSup, 
			        Color colorInf, Color centreColor, boolean bChangeDim, 
			        int exitMode, String sFavicon, boolean bRemoveButtons, 
			        boolean bRoundedTop, boolean bRoundedBottom) {
		
		//Builds the canvas for the dialogue
		super (colorSup, colorInf, centreColor, bRoundedTop, bRoundedBottom);
		
		//If title specified, make the dialogue modal 
		if (sTitle != null) {
		   JDialog.setDefaultLookAndFeelDecorated (true);
		   dialogue = new JDialog (owner);
		   dialogue.setTitle (sTitle);
		   dialogue.setModal (true);
		   
		//No title => not modal and without title bar
		} else {
			   JDialog.setDefaultLookAndFeelDecorated (false); 
			   dialogue = new JDialog (owner);
			   dialogue.setUndecorated (true);
			   dialogue.setFocusableWindowState (false);
		}
		
		//Sets the exit mode. EXIT_ON_CLOSE supported.
		adaptador = null;
		changeExitMode (exitMode);
		
		//Sets rest of properties: exit mode, resizability, favicon...
		dialogue.setResizable (bChangeDim);
		if (bRemoveButtons)
		   removeButtons (dialogue);
		setWindow (dialogue);
		setCanvas();
		setIcon(sFavicon);
		
		//Prepares busy mouse pointer: links wait cursor with glass pane
		dialogue.getGlassPane().setCursor (Cursor.getPredefinedCursor (Cursor.WAIT_CURSOR));
	}
	
	/**
	 * Customised constructor for dialogues without rounded panels.	 * 
	 */
	public CanvasDialogue (String sTitle, Window owner, Color colorSup, 
	        Color colorInf, Color centreColor, boolean bChangeDim, 
	        int exitMode, String sFavicon, boolean bRemoveButtons) {
		this (sTitle, owner, colorSup, colorInf, centreColor, bChangeDim, 
	          exitMode, sFavicon, bRemoveButtons, false, false);
	}
	
	/**
	 * Retrieves the dialogue's root container.
	 * @return JDialog object.
	 */
	public JDialog getWindow () {
		return dialogue;
	}
	
	/**
	 * Sets the canvas for the present dialogue.
	 */
	public void setCanvas () {
		dialogue.setContentPane (getCanvas());
	}
	
	/**
	 * Removes the dialogue's canvas. The end result is an empty dialogue that can
	 * be used for further manipulation.
	 */
	protected void removeCanvas () {
		dialogue.getContentPane().remove (getCanvas());
	}
	
	/**
	 * Allows changing the action taking place when the "X" button is pressed.
	 * @param iExitMode New exit mode. JDialog.EXIT_ON_CLOSE constant supported.
	 */
	public void changeExitMode (int iExitMode) {
		if (iExitMode != JDialog.EXIT_ON_CLOSE) {
		   if (adaptador != null) {
			  dialogue.removeWindowListener (adaptador);
			  adaptador = null;
		   }
		   dialogue.setDefaultCloseOperation (iExitMode);
		   
		//implements the closing of the whole application if previously undefined
		} else if (adaptador == null) {
			      dialogue.setDefaultCloseOperation (JDialog.DO_NOTHING_ON_CLOSE);
			      adaptador = new WindowAdapter() {
					              public void windowClosing (WindowEvent e) {
					                  System.exit(0);
					          }};
			      dialogue.addWindowListener (adaptador);
	    }	
	}	
	
	/**
	 * Changes the mouse pointer to "busy" mode for user feedback.   
	 * @param bActivity True if there is some kind of activity. False if not.
	 */
    public void busy (boolean bActivity) {
		dialogue.getGlassPane().setVisible (bActivity);
	}
}